@@ -166,7 +166,7 @@ class Client:
                     '            GError *error = NULL;\n'
                     '\n'
                     '            /* Parse indication */\n'
-                    '            output = __${message_fullname_underscore}_indication_parse (message, &error);\n'
+                    '            output = ${message_fullname_underscore}_indication_parse (message, &error);\n'
                     '            if (!output) {\n'
                     '                g_warning ("Couldn\'t parse \'${message_name}\' indication: %s",\n'
                     '                           error ? error->message : "Unknown error");\n'
@@ -462,7 +462,7 @@ class Client:
                 '    }\n'
                 '\n'
                 '    /* Parse reply */\n'
-                '    output = __${message_fullname_underscore}_response_parse (reply, &error);\n'
+                '    output = ${message_fullname_underscore}_response_parse (reply, &error);\n'
                 '    if (!output)\n'
                 '        g_task_return_error (task, error);\n'
                 '    else\n'
