@@ -8,7 +8,7 @@ DESCRIPTION:
 PUBLIC CLASSES AND METHODS:
    cQMIProtocolServer
 
-Copyright (c) 2012, Code Aurora Forum. All rights reserved.
+Copyright (c) 2013, The Linux Foundation. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -17,10 +17,9 @@ modification, are permitted provided that the following conditions are met:
     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
-    * Neither the name of Code Aurora Forum nor
-      the names of its contributors may be used to endorse or promote
-      products derived from this software without specific prior written
-      permission.
+    * Neither the name of The Linux Foundation nor the names of its
+      contributors may be used to endorse or promote products derived from 
+      this software without specific prior written permission.
 
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
@@ -136,11 +135,24 @@ bool cQMIProtocolServer::Connect( LPCSTR pControlFile )
    else
    {
       // SMD
-
-      if (name.find( "QMUXD:" ) == 0)
+      std::string defaultPath = "/var/";
+      
+      // Such as "SOCKET:/dev/socket/,0"
+      if (name.find( "SOCKET:" ) == 0)
       {
          // Remove qualifier
+         name = name.substr( 7 );
+      }
+      // Such as "QMUXD:0"
+      else if (name.find( "QMUXD:" ) == 0)
+      {
          name = name.substr( 6 );
+         name = defaultPath + "," + name;
+      }
+      // Just channel ID, IE: "0"
+      else
+      {
+         name = defaultPath + "," + name;
       }
 
       mpConnection = new cSocket();
