@@ -19,6 +19,7 @@
  *
  * Copyright (C) 2012 Google Inc.
  * Copyright (C) 2012-2017 Aleksander Morgado <aleksander@aleksander.es>
+ * Copyright (c) 2022 Qualcomm Innovation Center, Inc.
  */
 
 #ifndef _LIBQMI_GLIB_QMI_ENUMS_NAS_H_
@@ -30,6 +31,8 @@
 
 /**
  * SECTION: qmi-enums-nas
+ * @title: NAS enumerations and flags
+ * @short_description: Enumerations and flags in the NAS service.
  *
  * This section defines enumerations and flags used in the NAS service
  * interface.
@@ -165,6 +168,58 @@ typedef enum { /*< since=1.0 >*/
  * @QMI_NAS_ACTIVE_BAND_TDSCDMA_D: TD-SCDMA Band D.
  * @QMI_NAS_ACTIVE_BAND_TDSCDMA_E: TD-SCDMA Band E.
  * @QMI_NAS_ACTIVE_BAND_TDSCDMA_F: TD-SCDMA Band F.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_1: NR5G Band 1. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_2: NR5G Band 2. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_3: NR5G Band 3. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_5: NR5G Band 5. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_7: NR5G Band 7. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_8: NR5G Band 8. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_12: NR5G Band 12. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_13: NR5G Band 13. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_14: NR5G Band 14. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_18: NR5G Band 18. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_20: NR5G Band 20. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_25: NR5G Band 25. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_26: NR5G Band 26. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_28: NR5G Band 28. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_29: NR5G Band 29. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_30: NR5G Band 30. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_34: NR5G Band 34. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_38: NR5G Band 38. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_39: NR5G Band 39. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_40: NR5G Band 40. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_41: NR5G Band 41. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_48: NR5G Band 48. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_50: NR5G Band 50. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_51: NR5G Band 51. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_53: NR5G Band 53. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_65: NR5G Band 65. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_66: NR5G Band 66. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_70: NR5G Band 70. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_71: NR5G Band 71. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_74: NR5G Band 74. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_75: NR5G Band 75. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_76: NR5G Band 76. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_77: NR5G Band 77. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_78: NR5G Band 78. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_79: NR5G Band 79. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_80: NR5G Band 80. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_81: NR5G Band 81. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_82: NR5G Band 82. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_83: NR5G Band 83. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_84: NR5G Band 84. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_85: NR5G Band 85. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_86: NR5G Band 86. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_257: NR5G Band 257. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_258: NR5G Band 258. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_259: NR5G Band 259. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_260: NR5G Band 260. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_261: NR5G Band 261. Since 1.32.
+ * @QMI_NAS_ACTIVE_BAND_NR5G_46: NR5G Band 46. Since 1.36
+ * @QMI_NAS_ACTIVE_BAND_NR5G_91: NR5G Band 91. Since 1.36
+ * @QMI_NAS_ACTIVE_BAND_NR5G_92: NR5G Band 92. Since 1.36
+ * @QMI_NAS_ACTIVE_BAND_NR5G_93: NR5G Band 93. Since 1.36
+ * @QMI_NAS_ACTIVE_BAND_NR5G_94: NR5G Band 94. Since 1.36
  *
  * Band classes.
  *
@@ -265,7 +320,59 @@ typedef enum { /*< since=1.0 >*/
     QMI_NAS_ACTIVE_BAND_TDSCDMA_C = 202,
     QMI_NAS_ACTIVE_BAND_TDSCDMA_D = 203,
     QMI_NAS_ACTIVE_BAND_TDSCDMA_E = 204,
-    QMI_NAS_ACTIVE_BAND_TDSCDMA_F = 205
+    QMI_NAS_ACTIVE_BAND_TDSCDMA_F = 205,
+    QMI_NAS_ACTIVE_BAND_NR5G_1 = 250,
+    QMI_NAS_ACTIVE_BAND_NR5G_2 = 251,
+    QMI_NAS_ACTIVE_BAND_NR5G_3 = 252,
+    QMI_NAS_ACTIVE_BAND_NR5G_5 = 253,
+    QMI_NAS_ACTIVE_BAND_NR5G_7 = 254,
+    QMI_NAS_ACTIVE_BAND_NR5G_8 = 255,
+    QMI_NAS_ACTIVE_BAND_NR5G_20 = 256,
+    QMI_NAS_ACTIVE_BAND_NR5G_28 = 257,
+    QMI_NAS_ACTIVE_BAND_NR5G_38 = 258,
+    QMI_NAS_ACTIVE_BAND_NR5G_41 = 259,
+    QMI_NAS_ACTIVE_BAND_NR5G_50 = 260,
+    QMI_NAS_ACTIVE_BAND_NR5G_51 = 261,
+    QMI_NAS_ACTIVE_BAND_NR5G_66 = 262,
+    QMI_NAS_ACTIVE_BAND_NR5G_70 = 263,
+    QMI_NAS_ACTIVE_BAND_NR5G_71 = 264,
+    QMI_NAS_ACTIVE_BAND_NR5G_74 = 265,
+    QMI_NAS_ACTIVE_BAND_NR5G_75 = 266,
+    QMI_NAS_ACTIVE_BAND_NR5G_76 = 267,
+    QMI_NAS_ACTIVE_BAND_NR5G_77 = 268,
+    QMI_NAS_ACTIVE_BAND_NR5G_78 = 269,
+    QMI_NAS_ACTIVE_BAND_NR5G_79 = 270,
+    QMI_NAS_ACTIVE_BAND_NR5G_80 = 271,
+    QMI_NAS_ACTIVE_BAND_NR5G_81 = 272,
+    QMI_NAS_ACTIVE_BAND_NR5G_82 = 273,
+    QMI_NAS_ACTIVE_BAND_NR5G_83 = 274,
+    QMI_NAS_ACTIVE_BAND_NR5G_84 = 275,
+    QMI_NAS_ACTIVE_BAND_NR5G_85 = 276,
+    QMI_NAS_ACTIVE_BAND_NR5G_257 = 277,
+    QMI_NAS_ACTIVE_BAND_NR5G_258 = 278,
+    QMI_NAS_ACTIVE_BAND_NR5G_259 = 279,
+    QMI_NAS_ACTIVE_BAND_NR5G_260 = 280,
+    QMI_NAS_ACTIVE_BAND_NR5G_261 = 281,
+    QMI_NAS_ACTIVE_BAND_NR5G_12 = 282,
+    QMI_NAS_ACTIVE_BAND_NR5G_25 = 283,
+    QMI_NAS_ACTIVE_BAND_NR5G_34 = 284,
+    QMI_NAS_ACTIVE_BAND_NR5G_39 = 285,
+    QMI_NAS_ACTIVE_BAND_NR5G_40 = 286,
+    QMI_NAS_ACTIVE_BAND_NR5G_65 = 287,
+    QMI_NAS_ACTIVE_BAND_NR5G_86 = 288,
+    QMI_NAS_ACTIVE_BAND_NR5G_48 = 289,
+    QMI_NAS_ACTIVE_BAND_NR5G_14 = 290,
+    QMI_NAS_ACTIVE_BAND_NR5G_13 = 291,
+    QMI_NAS_ACTIVE_BAND_NR5G_18 = 292,
+    QMI_NAS_ACTIVE_BAND_NR5G_26 = 293,
+    QMI_NAS_ACTIVE_BAND_NR5G_30 = 294,
+    QMI_NAS_ACTIVE_BAND_NR5G_29 = 295,
+    QMI_NAS_ACTIVE_BAND_NR5G_53 = 296,
+    QMI_NAS_ACTIVE_BAND_NR5G_46 = 297,
+    QMI_NAS_ACTIVE_BAND_NR5G_91 = 298,
+    QMI_NAS_ACTIVE_BAND_NR5G_92 = 299,
+    QMI_NAS_ACTIVE_BAND_NR5G_93 = 300,
+    QMI_NAS_ACTIVE_BAND_NR5G_94 = 301,
 } QmiNasActiveBand;
 
 /**
@@ -274,18 +381,18 @@ typedef enum { /*< since=1.0 >*/
  * @QMI_NAS_NETWORK_SERVICE_DOMAIN_CS: Circuit switched.
  * @QMI_NAS_NETWORK_SERVICE_DOMAIN_PS: Packet switched.
  * @QMI_NAS_NETWORK_SERVICE_DOMAIN_CS_PS: Circuit and packet switched.
- * @QMI_NAS_NETWORK_SERVICE_DOMAIN_UNKNOWN: Unknown service.
+ * @QMI_NAS_NETWORK_SERVICE_DOMAIN_CAMPED: Camped. Since 1.36.
  *
  * Network Service Domain.
  *
  * Since: 1.0
  */
 typedef enum { /*< since=1.0 >*/
-    QMI_NAS_NETWORK_SERVICE_DOMAIN_NONE    = 0x00,
-    QMI_NAS_NETWORK_SERVICE_DOMAIN_CS      = 0x01,
-    QMI_NAS_NETWORK_SERVICE_DOMAIN_PS      = 0x02,
-    QMI_NAS_NETWORK_SERVICE_DOMAIN_CS_PS   = 0x03,
-    QMI_NAS_NETWORK_SERVICE_DOMAIN_UNKNOWN = 0x04,
+    QMI_NAS_NETWORK_SERVICE_DOMAIN_NONE   = 0x00,
+    QMI_NAS_NETWORK_SERVICE_DOMAIN_CS     = 0x01,
+    QMI_NAS_NETWORK_SERVICE_DOMAIN_PS     = 0x02,
+    QMI_NAS_NETWORK_SERVICE_DOMAIN_CS_PS  = 0x03,
+    QMI_NAS_NETWORK_SERVICE_DOMAIN_CAMPED = 0x04,
 } QmiNasNetworkServiceDomain;
 
 /**
@@ -1164,13 +1271,31 @@ typedef enum { /*< since=1.10 >*/
 
 /**
  * QmiNasDLBandwidth:
- * @QMI_NAS_DL_BANDWIDTH_1_4: 1.4 MHz
- * @QMI_NAS_DL_BANDWIDTH_3: 3 MHz
- * @QMI_NAS_DL_BANDWIDTH_5: 5 MHz
- * @QMI_NAS_DL_BANDWIDTH_10: 10 MHz
- * @QMI_NAS_DL_BANDWIDTH_15: 15 MHz
- * @QMI_NAS_DL_BANDWIDTH_20: 20 MHz
- * @QMI_NAS_DL_BANDWIDTH_INVALID: Invalid
+ * @QMI_NAS_DL_BANDWIDTH_1_4: LTE 1.4 MHz
+ * @QMI_NAS_DL_BANDWIDTH_3: LTE 3 MHz
+ * @QMI_NAS_DL_BANDWIDTH_5: LTE 5 MHz
+ * @QMI_NAS_DL_BANDWIDTH_10: LTE 10 MHz
+ * @QMI_NAS_DL_BANDWIDTH_15: LTE 15 MHz
+ * @QMI_NAS_DL_BANDWIDTH_20: LTE 20 MHz
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_5: 5G NR 5MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_10: 5G NR 10MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_15: 5G NR 15MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_20: 5G NR 20MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_25: 5G NR 25MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_30: 5G NR 30MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_40: 5G NR 40MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_50: 5G NR 50Mhz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_60: 5G NR 60MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_80: 5G NR 80MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_90: 5G NR 90MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_100: 5G NR 100MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_200: 5G NR 200MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_400: 5G NR 400MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_GSM_02: GSM 0.2MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_TDSCDMA_16: TD-SCDMA 1.6MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_WCDMA_5: WCDMA 5MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_WCDMA_10: WCDMA 10MHz. Since 1.36
+ * @QMI_NAS_DL_BANDWIDTH_5GNR_70: 5G NR 70MHz. Since 1.36
  * @QMI_NAS_DL_BANDWIDTH_UNKNOWN: Unknown
  *
  * DL Bandwidth.
@@ -1178,14 +1303,32 @@ typedef enum { /*< since=1.10 >*/
  * Since: 1.16
  */
 typedef enum { /*< since=1.16 >*/
-    QMI_NAS_DL_BANDWIDTH_1_4      = 0,
-    QMI_NAS_DL_BANDWIDTH_3        = 1,
-    QMI_NAS_DL_BANDWIDTH_5        = 2,
-    QMI_NAS_DL_BANDWIDTH_10       = 3,
-    QMI_NAS_DL_BANDWIDTH_15       = 4,
-    QMI_NAS_DL_BANDWIDTH_20       = 5,
-    QMI_NAS_DL_BANDWIDTH_INVALID  = 6,
-    QMI_NAS_DL_BANDWIDTH_UNKNOWN  = 0xFF
+    QMI_NAS_DL_BANDWIDTH_1_4        = 0,
+    QMI_NAS_DL_BANDWIDTH_3          = 1,
+    QMI_NAS_DL_BANDWIDTH_5          = 2,
+    QMI_NAS_DL_BANDWIDTH_10         = 3,
+    QMI_NAS_DL_BANDWIDTH_15         = 4,
+    QMI_NAS_DL_BANDWIDTH_20         = 5,
+    QMI_NAS_DL_BANDWIDTH_5GNR_5     = 6,
+    QMI_NAS_DL_BANDWIDTH_5GNR_10    = 7,
+    QMI_NAS_DL_BANDWIDTH_5GNR_15    = 8,
+    QMI_NAS_DL_BANDWIDTH_5GNR_20    = 9,
+    QMI_NAS_DL_BANDWIDTH_5GNR_25    = 10,
+    QMI_NAS_DL_BANDWIDTH_5GNR_30    = 11,
+    QMI_NAS_DL_BANDWIDTH_5GNR_40    = 12,
+    QMI_NAS_DL_BANDWIDTH_5GNR_50    = 13,
+    QMI_NAS_DL_BANDWIDTH_5GNR_60    = 14,
+    QMI_NAS_DL_BANDWIDTH_5GNR_80    = 15,
+    QMI_NAS_DL_BANDWIDTH_5GNR_90    = 16,
+    QMI_NAS_DL_BANDWIDTH_5GNR_100   = 17,
+    QMI_NAS_DL_BANDWIDTH_5GNR_200   = 18,
+    QMI_NAS_DL_BANDWIDTH_5GNR_400   = 19,
+    QMI_NAS_DL_BANDWIDTH_GSM_02     = 20,
+    QMI_NAS_DL_BANDWIDTH_TDSCDMA_16 = 21,
+    QMI_NAS_DL_BANDWIDTH_WCDMA_5    = 22,
+    QMI_NAS_DL_BANDWIDTH_WCDMA_10   = 23,
+    QMI_NAS_DL_BANDWIDTH_5GNR_70    = 24,
+    QMI_NAS_DL_BANDWIDTH_UNKNOWN    = 0xFF,
 } QmiNasDLBandwidth;
 
 /**
