@@ -37,42 +37,6 @@
 
 /*****************************************************************************/
 
-gchar *
-qmi_helpers_str_hex (gconstpointer mem,
-                     gsize         size,
-                     gchar         delimiter)
-{
-    const guint8 *data = mem;
-    gsize         i;
-    gsize         j;
-    gsize         new_str_length;
-    gchar        *new_str;
-
-    /* Get new string length. If input string has N bytes, we need:
-     * - 1 byte for last NUL char
-     * - 2N bytes for hexadecimal char representation of each byte...
-     * - N-1 bytes for the separator ':'
-     * So... a total of (1+2N+N-1) = 3N bytes are needed... */
-    new_str_length =  3 * size;
-
-    /* Allocate memory for new array and initialize contents to NUL */
-    new_str = g_malloc0 (new_str_length);
-
-    /* Print hexadecimal representation of each byte... */
-    for (i = 0, j = 0; i < size; i++, j += 3) {
-        /* Print character in output string... */
-        snprintf (&new_str[j], 3, "%02X", data[i]);
-        /* And if needed, add separator */
-        if (i != (size - 1) )
-            new_str[j + 2] = delimiter;
-    }
-
-    /* Set output string */
-    return new_str;
-}
-
-/*****************************************************************************/
-
 gboolean
 qmi_helpers_check_user_allowed (uid_t    uid,
                                 GError **error)
@@ -520,6 +484,10 @@ qmi_helpers_get_transport_type (const gchar  *path,
     if (g_file_test (rpmsg_sysfs_path, G_FILE_TEST_EXISTS))
         return QMI_HELPERS_TRANSPORT_TYPE_QMUX;
 
+    /* Allow libqmi to connect directly to a unix domain socket with a specific file name */
+    if (g_strrstr (device_basename, QMI_QMUX_SOCKET_FILE_NAME))
+        return QMI_HELPERS_TRANSPORT_TYPE_QMUX;
+
     g_set_error (error, QMI_CORE_ERROR, QMI_CORE_ERROR_FAILED,
                  "unexpected port subsystem");
     return QMI_HELPERS_TRANSPORT_TYPE_UNKNOWN;
