@@ -81,7 +81,7 @@ static GOptionEntry entries[] = {
       "Just allocate or release a WMS client. Use with `--client-no-release-cid' and/or `--client-cid'",
       NULL
     },
-    { NULL }
+    { NULL, 0, 0, 0, NULL, NULL, NULL }
 };
 
 GOptionGroup *
