@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2012-2017 Aleksander Morgado <aleksander@aleksander.es>
+ * Copyright (C) 2012-2023 Aleksander Morgado <aleksander@aleksander.es>
  * Copyright (c) 2022 Qualcomm Innovation Center, Inc.
  */
 
@@ -63,9 +63,11 @@ static gboolean device_open_version_info_flag;
 static gboolean device_open_sync_flag;
 static gchar *device_open_net_str;
 static gboolean device_open_proxy_flag;
+#if QMI_MBIM_QMUX_SUPPORTED
 static gboolean device_open_qmi_flag;
 static gboolean device_open_mbim_flag;
 static gboolean device_open_auto_flag;
+#endif
 static gchar *client_cid_str;
 static gboolean client_no_release_cid_flag;
 static gboolean verbose_flag;
@@ -103,6 +105,7 @@ static GOptionEntry main_entries[] = {
       "Request to use the 'qmi-proxy' proxy",
       NULL
     },
+#if QMI_MBIM_QMUX_SUPPORTED
     { "device-open-qmi", 0, 0, G_OPTION_ARG_NONE, &device_open_qmi_flag,
       "Open a cdc-wdm device explicitly in QMI mode",
       NULL
@@ -115,6 +118,7 @@ static GOptionEntry main_entries[] = {
       "Open a cdc-wdm device in either QMI or MBIM mode (default)",
       NULL
     },
+#endif
     { "device-open-net", 0, 0, G_OPTION_ARG_STRING, &device_open_net_str,
       "Open device with specific link protocol and QoS flags",
       "[net-802-3|net-raw-ip|net-qos-header|net-no-qos-header]"
@@ -143,7 +147,7 @@ static GOptionEntry main_entries[] = {
       "Print version",
       NULL
     },
-    { NULL }
+    { NULL, 0, 0, 0, NULL, NULL, NULL }
 };
 
 static gboolean
@@ -231,7 +235,7 @@ static void
 print_version_and_exit (void)
 {
     g_print (PROGRAM_NAME " " PROGRAM_VERSION "\n"
-             "Copyright (C) 2012-2022 Aleksander Morgado\n"
+             "Copyright (C) 2012-2023 Aleksander Morgado\n"
              "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl-2.0.html>\n"
              "This is free software: you are free to change and redistribute it.\n"
              "There is NO WARRANTY, to the extent permitted by law.\n"
@@ -475,6 +479,34 @@ allocate_client_ready (QmiDevice *dev,
         return;
 #else
         break;
+#endif
+    case QMI_SERVICE_ATR:
+#if defined HAVE_QMI_SERVICE_ATR
+        qmicli_atr_run (dev, QMI_CLIENT_ATR (client), cancellable);
+        return;
+#else
+        break;
+#endif
+    case QMI_SERVICE_IMSP:
+#if defined HAVE_QMI_SERVICE_IMSP
+        qmicli_imsp_run (dev, QMI_CLIENT_IMSP (client), cancellable);
+        return;
+#else
+        break;
+#endif
+    case QMI_SERVICE_IMSA:
+#if defined HAVE_QMI_SERVICE_IMSA
+        qmicli_imsa_run (dev, QMI_CLIENT_IMSA (client), cancellable);
+        return;
+#else
+        break;
+#endif
+    case QMI_SERVICE_IMS:
+#if defined HAVE_QMI_SERVICE_IMS
+        qmicli_ims_run (dev, QMI_CLIENT_IMS (client), cancellable);
+        return;
+#else
+        break;
 #endif
     case QMI_SERVICE_UNKNOWN:
     case QMI_SERVICE_CTL:
@@ -484,7 +516,6 @@ allocate_client_ready (QmiDevice *dev,
     case QMI_SERVICE_QCHAT:
     case QMI_SERVICE_RMTFS:
     case QMI_SERVICE_TEST:
-    case QMI_SERVICE_IMS:
     case QMI_SERVICE_ADC:
     case QMI_SERVICE_CSD:
     case QMI_SERVICE_MFS:
@@ -496,9 +527,7 @@ allocate_client_ready (QmiDevice *dev,
     case QMI_SERVICE_RFSA:
     case QMI_SERVICE_CSVT:
     case QMI_SERVICE_QCMAP:
-    case QMI_SERVICE_IMSP:
     case QMI_SERVICE_IMSVT:
-    case QMI_SERVICE_IMSA:
     case QMI_SERVICE_COEX:
     case QMI_SERVICE_STX:
     case QMI_SERVICE_BIT:
@@ -510,6 +539,7 @@ allocate_client_ready (QmiDevice *dev,
     case QMI_SERVICE_FOTA:
     case QMI_SERVICE_PDS:
     case QMI_SERVICE_OMA:
+    case QMI_SERVICE_SSC:
     default:
         break;
     }
@@ -689,10 +719,12 @@ device_new_ready (GObject *unused,
         exit (EXIT_FAILURE);
     }
 
+#if QMI_MBIM_QMUX_SUPPORTED
     if (device_open_mbim_flag + device_open_qmi_flag + device_open_auto_flag > 1) {
         g_printerr ("error: cannot specify multiple mode flags to open device\n");
         exit (EXIT_FAILURE);
     }
+#endif
 
     /* Setup device open flags */
     if (device_open_version_info_flag)
@@ -701,10 +733,12 @@ device_new_ready (GObject *unused,
         open_flags |= QMI_DEVICE_OPEN_FLAGS_SYNC;
     if (device_open_proxy_flag)
         open_flags |= QMI_DEVICE_OPEN_FLAGS_PROXY;
+#if QMI_MBIM_QMUX_SUPPORTED
     if (device_open_mbim_flag)
         open_flags |= QMI_DEVICE_OPEN_FLAGS_MBIM;
     if (device_open_auto_flag || (!device_open_qmi_flag && !device_open_mbim_flag))
         open_flags |= QMI_DEVICE_OPEN_FLAGS_AUTO;
+#endif
     if (expect_indications)
         open_flags |= QMI_DEVICE_OPEN_FLAGS_EXPECT_INDICATIONS;
     if (device_open_net_str) {
@@ -933,6 +967,34 @@ parse_actions (void)
     }
 #endif
 
+#if defined HAVE_QMI_SERVICE_ATR
+    if (qmicli_atr_options_enabled ()) {
+        service = QMI_SERVICE_ATR;
+        actions_enabled++;
+    }
+#endif
+
+#if defined HAVE_QMI_SERVICE_IMSP
+    if (qmicli_imsp_options_enabled ()) {
+        service = QMI_SERVICE_IMSP;
+        actions_enabled++;
+    }
+#endif
+
+#if defined HAVE_QMI_SERVICE_IMSA
+    if (qmicli_imsa_options_enabled ()) {
+        service = QMI_SERVICE_IMSA;
+        actions_enabled++;
+    }
+#endif
+
+#if defined HAVE_QMI_SERVICE_IMS
+    if (qmicli_ims_options_enabled ()) {
+        service = QMI_SERVICE_IMS;
+        actions_enabled++;
+    }
+#endif
+
     /* Cannot mix actions from different services */
     if (actions_enabled > 1) {
         g_printerr ("error: cannot execute multiple actions of different services\n");
@@ -1008,6 +1070,18 @@ int main (int argc, char **argv)
 #endif
 #if defined HAVE_QMI_SERVICE_FOX
     g_option_context_add_group (context, qmicli_fox_get_option_group ());
+#endif
+#if defined HAVE_QMI_SERVICE_ATR
+    g_option_context_add_group (context, qmicli_atr_get_option_group ());
+#endif
+#if defined HAVE_QMI_SERVICE_IMSP
+    g_option_context_add_group (context, qmicli_imsp_get_option_group ());
+#endif
+#if defined HAVE_QMI_SERVICE_IMSA
+    g_option_context_add_group (context, qmicli_imsa_get_option_group ());
+#endif
+#if defined HAVE_QMI_SERVICE_IMS
+    g_option_context_add_group (context, qmicli_ims_get_option_group ());
 #endif
     g_option_context_add_group (context, qmicli_link_management_get_option_group ());
     g_option_context_add_group (context, qmicli_qmiwwan_get_option_group ());
